<?php
/**
 * Copyright © 2016 Filoblu S.r.l. All rights reserved.
 */
namespace FiloBlu\OrderAdditionalData\Model\Plugin;

/**
 * Plugin that extends OrderManagement. Save Additional Data on order place.
 */
 
 class OrderPlace{

	/**
     * @var \Magento\Customer\Model\CustomerFactory
     */ 
    protected $customerFactory;

	/**
     * @var \Magento\Catalog\Model\ProductFactory
     */ 
    protected $productFactory;

	/**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */  
    protected $scopeConfig;

	/**
     * @var \Magento\Quote\Model\Quote\Address\RateRequestFactory
     */ 
    protected $rateRequestFactory;

	/**
     * @var \WebShopApps\MatrixRate\Model\Carrier\MatrixrateFactory
     */ 
    protected $carrierFactory;

	/**
     * @var \Magento\Framework\DataObjectFactory
     */ 
    protected $objectFactory;

	/**
     * @var \Magento\Sales\Api\Data\OrderExtensionFactory
     */ 
    protected $orderExtensionFactory;

	/**
     * @var \Magento\Sales\Api\Data\OrderItemExtensionFactory
     */ 
    protected $orderItemExtensionFactory;

    /**
     * Init plugin
     *
     * @param \Magento\Customer\Model\CustomerFactory $customerFactory
     * @param \Magento\Catalog\Model\ProductFactory $productFactory
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Magento\Quote\Model\Quote\Address\RateRequestFactory $rateRequestFactory
     * @param \WebShopApps\MatrixRate\Model\Carrier\MatrixrateFactory $carrierFactory
     * @param \Magento\Framework\DataObjectFactory $objectFactory
     * @param \Magento\Sales\Api\Data\OrderExtensionFactory $orderExtensionFactory,
     * @param \Magento\Sales\Api\Data\OrderItemExtensionFactory $orderItemExtensionFactory
     */
	public function __construct(
        \Magento\Customer\Model\CustomerFactory $customerFactory,
        \Magento\Catalog\Model\ProductFactory $productFactory,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Quote\Model\Quote\Address\RateRequestFactory $rateRequestFactory,
        \WebShopApps\MatrixRate\Model\Carrier\MatrixrateFactory $carrierFactory,
        \Magento\Framework\DataObjectFactory $objectFactory,
        \Magento\Sales\Api\Data\OrderExtensionFactory $orderExtensionFactory,
        \Magento\Sales\Api\Data\OrderItemExtensionFactory $orderItemExtensionFactory
	){
        $this->customerFactory=$customerFactory;
        $this->productFactory=$productFactory;
        $this->scopeConfig = $scopeConfig;
        $this->rateRequestFactory = $rateRequestFactory;
        $this->carrierFactory = $carrierFactory;
        $this->objectFactory = $objectFactory;
        $this->orderExtensionFactory = $orderExtensionFactory;
        $this->orderItemExtensionFactory = $orderItemExtensionFactory;
	}

    /**
     * Save additional data before place.
     *
     * @param \Magento\Sales\Api\OrderManagementInterface $management
     * @param \Magento\Sales\Api\Data\OrderInterface  $order
     * @return \Magento\Sales\Api\Data\OrderInterface 
     */
    public function beforePlace(\Magento\Sales\Api\OrderManagementInterface $management, \Magento\Sales\Api\Data\OrderInterface $order){    
        $order_extension_attributes=$order->getExtensionAttributes();
        $order_extension = $order_extension_attributes ? $order_extension_attributes : $this->orderExtensionFactory->create();   
        $order_fields=$this->scopeConfig->getValue('oad/general/order_data_fields', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        $additional_data=array();       
        if($order_fields){
            $source_objects=array();
            $order_fields=unserialize($order_fields);      
            foreach($order_fields as $order_field){
                if($order->getCustomerId()>0 && $order_field["object_type"]=="customer" && !array_key_exists("customer", $source_objects)){
                    $source_objects["customer"]=$this->customerFactory->create()->load($order->getCustomerId());
                }
                if($order_field["object_type"]=="free_shipping" && !array_key_exists("free_shipping", $source_objects)){
                    $source_objects["free_shipping"]=$this->simulateShippingAmount($order);
                }
                $data_object=$source_objects[$order_field["object_type"]];
                if($data_object){
                    $additional_data[$order_field["fieldname"]]=$data_object->getData($order_field["object_field"]);
                }
            }
                    
        }
        $order_extension->setFbAdditionalData(array($additional_data));
        $order->setExtensionAttributes($order_extension);
        
        $item_fields=$this->scopeConfig->getValue('oad/general/item_data_fields', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        if($item_fields){
            $item_fields=unserialize($item_fields);

            $items=$order->getItems();   
            $source_item_objects=array("product" => array(), "category" => array());
            foreach($items as $key=>$order_item){
                $additional_data=array();
                $item_extension_attributes=$order_item->getExtensionAttributes();
                $item_extension = $item_extension_attributes ? $item_extension_attributes : $this->orderItemExtensionFactory->create();   
                foreach($item_fields as $item_field){
                    if($item_field["object_type"]=="product"){
                        $product_id=$order_item->getProductId();
                        if(!array_key_exists($product_id, $source_item_objects["product"])){
                            $source_item_objects["product"][$product_id]=$this->productFactory->create()->load($product_id);
                        }
                        $data_object=$source_item_objects["product"][$product_id];                     
                    }                    
                    $additional_data[$item_field["fieldname"]] = $data_object->getData($item_field["object_field"]);                    
                } 
                $item_extension->setFbAdditionalData(array($additional_data));
                $order_item->setExtensionAttributes($item_extension);
                $items[$key]=$order_item;
            }
            $order->setItems($items);
        }
        return array($order);
    }

    /**
     * Simulate Shiipping Costs in case of Free Shipping.
     *
     * @param \Magento\Sales\Api\Data\OrderInterface  $order
     * @return \Magento\Framework\DataObject
     */
    protected function simulateShippingAmount(\Magento\Sales\Api\Data\OrderInterface $order) {        
        
        $shipping_amounts = array(0);

        // matrix rate
        if ($order->getShippingAmount()==0) {
            
            $address = $order->getShippingAddress();
            $store=$order->getStore();
            $request = $this->rateRequestFactory->create();
            $request->setDestCountryId($address->getCountryId());
            $request->setDestRegionId($address->getRegionId());
            $request->setDestPostcode($address->getPostcode());
            $request->setDestCity($address->getCity());
            $request->setPackageValue($order->getBaseSubtotal());
            $request->setPackageValueWithDiscount($order->getBaseSubtotal());
            $request->setPackageWeight($order->getWeight());
            $request->setFreeMethodWeight($order->getWeight());
            $request->setPackageQty($order->getTotalQtyOrdered());
            $request->setStoreId($store->getId());
            $request->setWebsiteId($store->getWebsiteId());
            $request->setBaseCurrency($order->getBaseCurrencyCode());
            $request->setPackageCurrency($order->getOrderCurrencyCode());
            $request->setLimitCarrier(null);
            $request->setFreeShipping(null);
            
            //get rates for the matrixrate request created
            $matrixrate = $this->carrierFactory->create();
            //$rates = $matrixrate->getRate($request, false);
            $result = $matrixrate->collectRates($request);
            if ($result) {
                $rates = $result->getAllRates();
                if (!empty($rates)) {
                    foreach ($rates as $rate) {
                        $price=$rate->getPrice();
                        if (isset($price) && $price > 0) {
                            $shipping_amounts[] = $price;
                        }
                    }
                }
            }


        }
        $shippingObject = $this->objectFactory->create();
        $shippingObject->setData("value", max($shipping_amounts));
        return $shippingObject;
    }    
}